///////////////////////////
/// EXTERNAL PARAMETERS ///
///////////////////////////

Include "cube.par";


/////////////////
/// REGIONS ///
////////////////

Group
{
/// Regions ///
Omega_C = Region[{cubeID}]; Gamma_C = Region[{cubeBoundaryID}];
Omega_M = Region[{coilID}]; // external Magnet
Omega_CC = Region[{airID, Omega_M}]; Gamma_CC = Region[{airBoundaryID}];

/// Domains (set of regions) ///
Omega = Region[{Omega_C, Omega_CC}];
Gamma_Gauge = Region[{Gamma_CC}];
}


///////////////////
/// FUNCTIONS ///
///////////////////

Function
{

/// Screening function parameter:
DefineConstant[frequency = {50, Name "/Inputs/1Parameters/Input signal: frequency (Hz)"}];
pulsation = 2*Pi*frequency;

/// Time loop adaptive ///
DefineConstant[initialTimeStep = {1e-4, Name "/Inputs/3AdptiveTimeScheme/Initial time step size (s)"}];
//DefineConstant[minimumTimeStep = {1e-6, Name "/Inputs/3AdptiveTimeScheme/Minimum time step size (s)"}];
//DefineConstant[maximumTimeStep = {2e-4, Name "/Inputs/3AdptiveTimeScheme/Maximum time step size (s)"}];
//DefineConstant[method = {"Euler", Name "/Inputs/3AdptiveTimeScheme/Integration method"}]; // Euler, Trapezoidal, Gear_x = backward differentiating methods of order x (1 to 6)
//DefineConstant[relativeTolerance = {1e-8, Name "/Inputs/3AdptiveTimeScheme/Relative tolerance on solution"}];
//DefineConstant[absoluteTolerance = {1e-8, Name "/Inputs/3AdptiveTimeScheme/Absolute tolerance on solution"}];
//breakPoints = {}; // List of time points to be met [s]
/// Resampling time ///
DefineConstant[resamplingTimeStep = {1/(100*frequency), Name "/Inputs/3AdptiveTimeScheme/resampling data"}];
DefineConstant[initialTime = {0, Name "/Inputs/1Parameters/Generic scheme: Initial time (s)"}];
DefineConstant[finalTime = {0.1/frequency, Name "/Inputs/1Parameters/Generic scheme: Initial time (s)"}];
globalTime = finalTime-initialTime;

/// Time loop theta ///
DefineConstant[theta = {1, Name "/Inputs/2ThetaScheme/Theta value [0 = explicit Euler, 0.5 = Crank-Nicholson, 1 = implicit Euler]"}];
DefineConstant[initialTimeThetaLoop = {initialTime, Name "/Inputs/2ThetaScheme/Initial time of theta scheme (s)"}];
DefineConstant[endTimeThetaLoop = {initialTime+initialTimeStep, Name "/Inputs/2ThetaScheme/Final time of theta scheme (s)"}];
DefineConstant[timeStepThetaLoop = {initialTimeStep, Name "/Inputs/2ThetaScheme/time step for theta scheme (s)"}];
DefineConstant[numberOfTimeSteps = {globalTime/initialTimeStep, Name "/Inputs/2ThetaScheme/Euler: number of time steps (-)"}];

/// Physical constants ///
mu0 = 4*Pi*1e-7; mu0 = mu0/scaling;
mur = 1;
mu = mur*mu0;
nu = 1/mu;

jc = jc/scaling^2;

/// Ramping up with plateau (includes a smoothing function for the first few time steps): 
//timeFunction[] = (2/(Exp[(-1)*alpha*pulsation*$Time]+1)-1)*Sin[omega*$Time];
timeFunction[] = Sin[pulsation*$Time];

/// Equivalent electrical conductivity:
sigma[Omega_C] = 1e6; // $1 = Curl(H), nonlinear case

/// Ampere-turns in coil:
Je[Omega_M] = 1e6*Unit[Vector[-Y[], X[], 0]]; // Counter-clockwise
}


/////////////////////
/// INTEGRATON ///
////////////////////

Include "./integration.par";


//////////////////
/// JACOBIAN ///
//////////////////

Include "./jacobian.par";


//////////////////////
/// CONSTRAINTS ///
//////////////////////

Constraint
{
 { Name AGauge; Type Assign; Case { { Region Omega; SubRegion Gamma_Gauge; Value 0.; } } }
 { Name AConstraint; Type Assign; Case { { Region Gamma_CC; Value 0.; } } }
 { Name VConstraint; Type Assign; Case { { Region Gamma_C; Value 0.; } } }
 //{ Name phiConstraint; Type Assign; Case { { Region Gamma_Gauge; Value 0.; } } }
}


/////////////////////////
/// FUNCTION SPACE ///
/////////////////////////

FunctionSpace
{
 {
  Name A_FunctionSpace; Type Form1;
  BasisFunction
  { 
   { Name wa_Edge; NameOfCoef ca_Edge;  Function BF_Edge; Support Omega; Entity EdgesOf[All, Not Gamma_C]; }
   { Name wa_Edge2 ; NameOfCoef ca_Edge2; Function BF_Edge; Support Omega; Entity EdgesOf[Gamma_C]; }
  }
  Constraint
  {
   { NameOfCoef ca_Edge; EntityType EdgesOf; NameOfConstraint AConstraint; }
   { NameOfCoef ca_Edge; EntityType EdgesOfTreeIn; EntitySubType StartingOn; NameOfConstraint AGauge; }
   { NameOfCoef ca_Edge2; EntityType EdgesOf; NameOfConstraint AConstraint; }
  }
 }
 {
  Name V_FunctionSpace; Type Form0;
  BasisFunction
  {
   { Name wv_Node; NameOfCoef cv_Node; Function BF_GroupOfNodes; Support Omega_C; Entity GroupsOfNodesOf[Gamma_C]; }
  }
  Constraint
  {
   { NameOfCoef cv_Node; EntityType NodesOf; NameOfConstraint VConstraint; }
  }
 }
/*
 { // a.w = 0 gauge condition, w is an arbitrary radiative field written as for instance, w = Grad(Phi), where Phi is an arbitrary scalar potential. w.n on boundary should be zero at infinity or dPhi = 0 (Phi constante = 0 in our case).
  Name Phi_FunctionSpace; Type Form0;
  BasisFunction
  {
   { Name wp_Node; NameOfCoef cp_Node; Function BF_Node; Support Omega; Entity NodesOf[All]; }
  }
  Constraint
  {
   { NameOfCoef cp_Node; EntityType NodesOf; NameOfConstraint phiConstraint; }
  }
 }
*/
}


//////////////////////
/// FORMULATION ///
//////////////////////

Formulation
{
 {
  Name AV_Formulation; Type FemEquation;
  Quantity
  {
   { Name A; Type Local; NameOfSpace A_FunctionSpace; }
   { Name V; Type Local; NameOfSpace V_FunctionSpace; }
  // { Name Phi; Type Local; NameOfSpace Phi_FunctionSpace; }
  }	
  Equation
  {
    // Curl(nu*Curl(A))+sigma*(dt(A)+Grad(V))-nu*Ba = 0, H = Ha(t)+Hi, B = Curl(Ai+Aa) = Bi+Ba, in Omega = Omega_C+Omega_CC, curl so vector field
   Galerkin { [nu*Dof{d A}, {d A}]; In Omega; Jacobian jacobianVolume; Integration basicIntegration; }
   Galerkin { DtDof[sigma[]*Dof{d A}, {A}]; In Omega_C; Jacobian jacobianVolume; Integration basicIntegration; }
   Galerkin { [sigma[]*Dof{d V}, {A}]; In Omega_C; Jacobian jacobianVolume; Integration basicIntegration; }  
   Galerkin { [-Je[], {A}]; In Omega_M; Jacobian jacobianVolume; Integration basicIntegration; }
   // div(J) = 0, in Omega_C, divergence so scalar potential
   Galerkin { DtDof[sigma[]*Dof{A}, {d V}]; In Omega_C; Jacobian jacobianVolume; Integration basicIntegration; }
   Galerkin { [sigma[]*Dof{d V}, {d V}]; In Omega_C; Jacobian jacobianVolume; Integration basicIntegration; }
   // Coulomb gauge: div(A) = 0, I can choose whatever "Phi" to satisfy Coulomb gauge but it should have an associated boundary condition and be solved.
/*
  // a.w = 0 gauge condition, where w = Grad(Phi), Phi is arbitrary as of the radiative field w. We choose Phi so that Phi = 0 on the boundary Omega and we apply the integral to the basis functions of a and Phi since we need to solve for Phi as well.
   Galerkin { [Dof{A}, {d Phi}]; In Omega; Jacobian jacobianVolume; Integration basicIntegration; }
   Galerkin { [Dof{d Phi}, {A}]; In Omega; Jacobian jacobianVolume; Integration basicIntegration; }
*/
  }
 }
}


//////////////////
/// RESOLUTION ///
//////////////////


/// Non-linear iterative scheme to compute the magnetic field H:
/// Classic Newton algorithm:
Resolution
{
 {
  Name resolution;
  System
  {
   { Name AV_System; NameOfFormulation AV_Formulation; }
  }
  Operation 
  {
   CreateDirectory["Results"];
   InitSolution[AV_System]; SaveSolution[AV_System];
   TimeLoopTheta[initialTimeThetaLoop, finalTime, timeStepThetaLoop, theta]
   {
    Generate[AV_System];
    Solve[AV_System];
    SaveSolution[AV_System];
   }
  }
 }
 {
  Name complexResolution;
  System
  {
   { Name AV_System; NameOfFormulation AV_Formulation; Type ComplexValue; Frequency frequency; }
  }
  Operation
  {
   Generate[AV_System]; Solve[AV_System]; SaveSolution[AV_System];
  }
 }
}


////////////////////////
/// Onelab interface ///
////////////////////////



//////////////////////
/// POSTPROCESSING ///
//////////////////////

PostProcessing
{
 {
  Name AV_PostProcessing; NameOfFormulation AV_Formulation; NameOfSystem AV_System;
  Quantity
  {
   {Name b; Value{Local{[{d A}/scaling^2]; In Omega; Jacobian jacobianVolume;}}}
   {Name j; Value{Local{[-sigma[]*({d A}+{d V})]; In Omega; Jacobian jacobianVolume;}}}
   {Name e; Value{Local{[-({d A}+{d V})]; In Omega; Jacobian jacobianVolume;}}}
   {Name ja; Value{Local{[Je[]] ; In Omega_M; Jacobian jacobianVolume;}}}
  }
 }
}

PostOperation
{
 {
  Name AV_PostOperation; NameOfPostProcessing AV_PostProcessing;
  Operation
  {
   Print[b, OnElementsOf Omega_C, File "./Results/vectorB.pos", Name "B (T)"];
   Print[e, OnElementsOf Omega_C, File "./Results/vectorE.pos", Name "E (V/m)"];
   Print[j, OnElementsOf Omega_C, File "./Results/vectorJ.pos", Name "J (A/m2)"];
   Print[b, OnPoint {cubeLength/2, cubeLength/2, cubeLength/2}, File "./Results/vectorBAtCorner.pos", Name "B (T)"];
   Print[b, OnPoint {0, 0, 0}, File "./Results/vectorBAtCenter.pos", Name "B (T)"];
   Print[ja, OnElementsOf Omega_M, File "./Results/vectorJa.pos"];
  }
 }
}

Printf("Number of time steps [-]: %.0f", numberOfTimeSteps);
// Truncature local ne marche pas -> utilizer residue de Galerkin
